"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk_1 = require("chalk");
const cli_framework_1 = require("@ionic/cli-framework");
const constants_1 = require("../constants");
const guards_1 = require("../guards");
const IONIC_LOGO = String.raw `
   _             _
  (_) ___  _ __ (_) ___
  | |/ _ \| '_ \| |/ __|
  | | (_) | | | | | (__
  |_|\___/|_| |_|_|\___|`;
const COMMAND_DECORATIONS = [
    [constants_1.CommandGroup.Beta, chalk_1.default.red.bold('(beta)')],
    [constants_1.CommandGroup.Deprecated, chalk_1.default.yellow.bold('(deprecated)')],
];
const NAMESPACE_DECORATIONS = [
    [constants_1.NamespaceGroup.Beta, chalk_1.default.red.bold('(beta)')],
    [constants_1.NamespaceGroup.Deprecated, chalk_1.default.yellow.bold('(deprecated)')],
];
function showHelp(env, inputs) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const location = yield env.namespace.locate(inputs);
        if (guards_1.isCommand(location.obj)) {
            const formatter = new CommandHelpFormatter({ location, command: location.obj });
            env.log.rawmsg(yield formatter.format());
        }
        else {
            if (location.args.length > 0) {
                env.log.error(`Unable to find command: ${chalk_1.default.green(inputs.join(' '))}` +
                    (env.project.directory ? '' : '\nYou may need to be in an Ionic project directory.'));
            }
            const isLoggedIn = yield env.session.isLoggedIn();
            const now = new Date();
            const prefix = isLoggedIn ? chalk_1.default.blue('PRO') + ' ' : '';
            const version = env.plugins.ionic.meta.pkg.version;
            const suffix = now.getMonth() === 9 && now.getDate() === 31 ? ' 🎃' : '';
            const formatter = new NamespaceHelpFormatter({
                inProject: env.project.directory ? true : false,
                version: prefix + version + suffix,
                location,
                namespace: location.obj,
            });
            env.log.rawmsg(yield formatter.format());
        }
    });
}
exports.showHelp = showHelp;
class NamespaceHelpFormatter extends cli_framework_1.NamespaceHelpFormatter {
    constructor(_a) {
        var { version, inProject } = _a, rest = tslib_1.__rest(_a, ["version", "inProject"]);
        super(rest);
        this.inProject = inProject;
        this.version = version;
    }
    formatHeader() {
        const _super = name => super[name];
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.namespace.parent ? _super("formatHeader").call(this) : this.formatIonicHeader();
        });
    }
    formatIonicHeader() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return IONIC_LOGO + `  CLI ${this.version}\n\n`;
        });
    }
    formatBeforeNamespaceDescription(meta) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return formatGroupDecorations(NAMESPACE_DECORATIONS, meta.groups);
        });
    }
    formatBeforeCommandDescription(cmd) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return formatGroupDecorations(COMMAND_DECORATIONS, cmd.groups);
        });
    }
    getExtraOptions() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return ['--verbose', '--quiet', '--no-interactive', '--no-color', '--confirm'];
        });
    }
    formatCommands() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { strong } = this.colors;
            const commands = yield this.namespace.getCommandMetadataList();
            const globalCmds = commands.filter(cmd => cmd.type === 'global');
            const projectCmds = commands.filter(cmd => cmd.type === 'project');
            return ((yield this.formatCommandGroup('Global Commands', globalCmds)) +
                (this.inProject ? yield this.formatCommandGroup('Project Commands', projectCmds) : `\n  ${strong('Project Commands')}:\n\n    You are not in a project directory.\n`));
        });
    }
    filterCommandCallback(cmd) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const ns = yield cmd.namespace.getMetadata();
            return (!cmd.groups || !cmd.groups.includes(constants_1.CommandGroup.Hidden)) && (!ns.groups || !ns.groups.includes(constants_1.NamespaceGroup.Hidden));
        });
    }
}
class CommandHelpFormatter extends cli_framework_1.CommandHelpFormatter {
    formatBeforeOptionDescription(opt) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { weak } = this.colors;
            return opt.hint ? `${weak(`[${opt.hint}]`)} ` : '';
        });
    }
    formatOptions() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const metadata = yield this.getCommandMetadata();
            const options = metadata.options ? metadata.options : [];
            const basicOptions = options.filter(o => !o.groups || !o.groups.includes(constants_1.OptionGroup.Advanced));
            const advancedOptions = options.filter(o => o.groups && o.groups.includes(constants_1.OptionGroup.Advanced));
            return ((yield this.formatOptionsGroup('Options', basicOptions)) +
                (yield this.formatOptionsGroup('Advanced Options', advancedOptions)));
        });
    }
    filterOptionCallback(opt) {
        return !opt.groups || !opt.groups.includes(constants_1.OptionGroup.Hidden);
    }
}
function formatGroupDecorations(decorations, groups) {
    if (!groups) {
        return '';
    }
    const prepends = decorations.filter(([g]) => groups.includes(g)).map(([, d]) => d);
    return prepends.length ? prepends.join(' ') + ' ' : '';
}
