"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path = require("path");
const util = require("util");
const chalk_1 = require("chalk");
const Debug = require("debug");
const lodash = require("lodash");
const array_1 = require("@ionic/cli-framework/utils/array");
const constants_1 = require("../constants");
const errors_1 = require("./errors");
const debug = Debug('ionic:cli-utils:lib:hooks');
exports.HOOKS_PKG = '@ionic/cli-hooks';
exports.ADD_CORDOVA_ENGINE_HOOK = path.join('node_modules', exports.HOOKS_PKG, 'add-cordova-engine.js');
exports.REMOVE_CORDOVA_ENGINE_HOOK = path.join('node_modules', exports.HOOKS_PKG, 'remove-cordova-engine.js');
class Hook {
    get script() {
        return `ionic:${this.name}`;
    }
    constructor({ config, project, shell }) {
        this.config = config;
        this.project = project;
        this.shell = shell;
    }
    run(input) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { pkgManagerArgs } = yield Promise.resolve().then(() => require('./utils/npm'));
            const type = this.project.type;
            if (!type || !this.project.directory) {
                return; // TODO: will we need hooks outside a project?
            }
            const project = yield this.project.load();
            const pkg = yield this.project.loadPackageJson();
            const config = yield this.config.load();
            const { npmClient } = config;
            debug(`Looking for ${chalk_1.default.cyan(this.script)} npm script.`);
            if (pkg.scripts && pkg.scripts[this.script]) {
                debug(`Invoking ${chalk_1.default.cyan(this.script)} npm script.`);
                const [pkgManager, ...pkgArgs] = yield pkgManagerArgs({ npmClient, shell: this.shell }, { command: 'run', script: this.script });
                yield this.shell.run(pkgManager, pkgArgs, {});
            }
            const hooks = array_1.conform(project.hooks[this.name]);
            for (const h of hooks) {
                const p = path.resolve(this.project.directory, h);
                try {
                    if (path.extname(p) !== '.js') {
                        throw new Error(`Hooks must be .js files with a function for its default export.`);
                    }
                    const hook = yield this.loadHookFn(p);
                    if (!hook) {
                        throw new Error(`Module must have a function for its default export.`);
                    }
                    yield hook(lodash.assign({}, input, {
                        project: {
                            type,
                            dir: this.project.directory,
                            srcDir: yield this.project.getSourceDir(),
                        },
                        argv: process.argv,
                        env: process.env,
                    }));
                }
                catch (e) {
                    throw new errors_1.HookException(`An error occurred while running an Ionic CLI hook defined in ${chalk_1.default.bold(constants_1.PROJECT_FILE)}.\n` +
                        `Hook: ${chalk_1.default.bold(this.name)}\n` +
                        `File: ${chalk_1.default.bold(p)}\n\n` +
                        `${chalk_1.default.red(e.stack ? e.stack : e)}`);
                }
            }
        });
    }
    loadHookFn(p) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const module = require(p);
            if (typeof module === 'function') {
                return module;
            }
            else if (typeof module.default === 'function') {
                return module.default;
            }
            const inspection = util.inspect(module, { colors: chalk_1.default.enabled });
            debug(`Could not load hook function ${chalk_1.default.bold(p)}: ${inspection} not a function`);
        });
    }
}
exports.Hook = Hook;
function addHook(baseDir, hooks, hook) {
    const hookPaths = array_1.conform(hooks);
    const resolvedHookPaths = hookPaths.map(p => path.resolve(baseDir, p));
    if (!resolvedHookPaths.includes(path.resolve(baseDir, hook))) {
        hookPaths.push(hook);
    }
    return hookPaths;
}
exports.addHook = addHook;
function removeHook(baseDir, hooks, hook) {
    const hookPaths = array_1.conform(hooks);
    const i = locateHook(baseDir, hookPaths, hook);
    if (i >= 0) {
        hookPaths.splice(i, 1);
    }
    return hookPaths;
}
exports.removeHook = removeHook;
function locateHook(baseDir, hooks, hook) {
    return array_1.conform(hooks).map(p => path.resolve(baseDir, p)).indexOf(path.resolve(baseDir, hook));
}
exports.locateHook = locateHook;
