"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk_1 = require("chalk");
const Debug = require("debug");
const lodash = require("lodash");
const through2 = require("through2");
const split2 = require("split2");
const cli_framework_1 = require("@ionic/cli-framework");
const format_1 = require("@ionic/cli-framework/utils/format");
const process_1 = require("@ionic/cli-framework/utils/process");
const generate_1 = require("../../generate");
const errors_1 = require("../../errors");
const ANGULAR_SCHEMATICS_PACKAGE = '@schematics/angular';
const IONIC_SCHEMATICS_PACKAGE = '@ionic/schematics-angular';
const debug = Debug('ionic:cli-utils:lib:project:angular:generate');
const ANGULAR_SCHEMATICS_BLACKLIST = ['application', 'appShell', 'universal'];
const ANGULAR_SCHEMATICS_SIMPLE = ['class', 'interface', 'module', 'enum'];
function pluralizeGeneratorType(type) {
    const suffix = type === 'class' ? 'es' : 's';
    return `${type}${suffix}`;
}
function generateAliasMap(schematics) {
    return new Map(lodash.flatten(schematics.map(s => [...s.aliases.map(a => [a, s.type])])));
}
function extractSchematicsFromCollection(collection) {
    return lodash.toPairs(collection.description.schematics)
        .filter(([k, v]) => !ANGULAR_SCHEMATICS_BLACKLIST.includes(k))
        .map(([type, v]) => ({ type, description: v.description, collection: collection.description.name, aliases: v.aliases || [] }));
}
function buildPathForGeneratorType(type, name) {
    if (ANGULAR_SCHEMATICS_SIMPLE.includes(type)) {
        return name;
    }
    if (name.includes('/')) {
        return name;
    }
    return `${pluralizeGeneratorType(type)}/${name}`;
}
exports.buildPathForGeneratorType = buildPathForGeneratorType;
class GenerateRunner extends generate_1.GenerateRunner {
    getCommandMetadata() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const schematics = yield this.getSchematics();
            const schematicNames = schematics.map(s => s.type);
            const schematicNamesAndAliases = [...schematicNames, ...lodash.flatten(schematics.map(s => s.aliases))];
            return {
                groups: [],
                description: `Generate Angular classes such as pages, components, directives, services, etc.`,
                longDescription: `
Automatically create components for your Ionic app.

This command uses the Angular CLI to generate components.

 - List generators with the ${chalk_1.default.green('--list')} option.
 - For a detailed list of options for each generator, use ${chalk_1.default.green('ng generate <type> --help')}.
 - For ${schematics.filter(s => s.collection === IONIC_SCHEMATICS_PACKAGE).map(t => chalk_1.default.green(t.type)).join(', ')} types, use ${chalk_1.default.green('ng generate <type> --help --collection @ionic/schematics-angular')}.

${chalk_1.default.green('ionic generate')} is more opinionated than ${chalk_1.default.green('ng generate')}. Aside from simpler generator types (${ANGULAR_SCHEMATICS_SIMPLE.map(t => chalk_1.default.green(pluralizeGeneratorType(t))).join(', ')}), generated files are placed in ${chalk_1.default.bold('src/app/<type>/<name>/')}. See the CLI documentation${chalk_1.default.cyan('[1]')} for an overview of recommended project structure.

Remember, you can use slashes in ${chalk_1.default.green('name')} to nest components deeper, but you must specify the full path within ${chalk_1.default.bold('src/app/')}. For example, specify a name of ${chalk_1.default.green('pages/tabs-page/tab1')} to generate page files at ${chalk_1.default.bold('src/app/pages/tabs-page/tab1/')}.

To test a generator before file modifications are made, use the ${chalk_1.default.green('--dry-run')} option.

${chalk_1.default.cyan('[1]')}: ${chalk_1.default.bold('https://ionicframework.com/docs/cli/projects.html#project-structure')}
      `,
                exampleCommands: [
                    '--list',
                    '-d',
                    'page',
                    'page contact',
                    'component pages/contact/form',
                    'component form --change-detection OnPush',
                    'directive ripple --skip-import',
                    's services/api/user',
                ],
                inputs: [
                    {
                        name: 'type',
                        description: `The type of generator (e.g. ${schematics.slice(0, 4).map(t => chalk_1.default.green(t.type)).join(', ')}; use ${chalk_1.default.green('--list')} to see all)`,
                        validators: [cli_framework_1.validators.required, cli_framework_1.contains(schematicNamesAndAliases, {})],
                    },
                    {
                        name: 'name',
                        description: 'The name of the component being generated',
                        validators: [cli_framework_1.validators.required],
                    },
                ],
                options: [
                    {
                        name: 'list',
                        description: 'List available generators',
                        type: Boolean,
                        aliases: ['l'],
                    },
                    {
                        name: 'dry-run',
                        description: 'Run generate without making any file changes',
                        type: Boolean,
                        aliases: ['d'],
                    },
                    {
                        name: 'force',
                        description: 'Force overwriting of files',
                        type: Boolean,
                        aliases: ['f'],
                    },
                ],
            };
        });
    }
    ensureCommandLine(inputs, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const schematics = yield this.getSchematics();
            if (schematics.length === 0) {
                throw new errors_1.FatalException(`No suitable schematics found.`);
            }
            if (options['list']) {
                const columnHeaders = ['name', 'alias', 'description', 'collection'];
                this.env.log.rawmsg(format_1.columnar(schematics.map(({ type, description, collection, aliases }) => [chalk_1.default.green(type), aliases.map(a => chalk_1.default.green(a)).join(', '), description, chalk_1.default.green(collection)]), { columnHeaders }));
                throw new errors_1.FatalException('', 0);
            }
            if (!inputs[0]) {
                const type = yield this.env.prompt({
                    type: 'list',
                    name: 'type',
                    message: 'What would you like to generate:',
                    choices: schematics.map(t => t.type),
                });
                inputs[0] = type;
            }
            if (!inputs[1]) {
                const name = yield this.env.prompt({
                    type: 'input',
                    name: 'name',
                    message: 'What should the name be?',
                    validate: v => cli_framework_1.validators.required(v),
                });
                inputs[1] = name;
            }
        });
    }
    createOptionsFromCommandLine(inputs, options) {
        const baseOptions = super.createOptionsFromCommandLine(inputs, options);
        return Object.assign({}, lodash.omit(options, '_', '--', 'dry-run', 'd', 'force', 'f'), baseOptions, { force: options['force'] ? true : false, dryRun: options['dry-run'] ? true : false });
    }
    run(options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const schematics = yield this.getSchematics();
            const aliases = generateAliasMap(schematics);
            const type = aliases.get(options.type) || options.type;
            const schematic = schematics.find(s => s.type === type);
            if (!schematic) {
                throw new errors_1.FatalException(`Unknown generator type: ${chalk_1.default.green(type)}`);
            }
            const name = buildPathForGeneratorType(type, options.name);
            debug(`${options.type} resolved to ${type}`);
            debug(`${options.name} normalized to ${name}`);
            try {
                yield this.generateComponent(schematic, name, lodash.omit(options, 'type', 'name'));
            }
            catch (e) {
                debug(e);
                throw new errors_1.FatalException(`Could not generate ${chalk_1.default.green(type)}.`);
            }
            if (!options.dryRun) {
                this.env.log.ok(`Generated ${chalk_1.default.green(type)}!`);
            }
        });
    }
    getSchematics() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!this.schematics) {
                try {
                    const { SchematicEngine } = yield Promise.resolve().then(() => require('@angular-devkit/schematics'));
                    const { NodeModulesEngineHost } = yield Promise.resolve().then(() => require('@angular-devkit/schematics/tools'));
                    const engineHost = new NodeModulesEngineHost();
                    const engine = new SchematicEngine(engineHost);
                    this.schematics = [
                        ...extractSchematicsFromCollection(engine.createCollection(IONIC_SCHEMATICS_PACKAGE)),
                        ...extractSchematicsFromCollection(engine.createCollection(ANGULAR_SCHEMATICS_PACKAGE)),
                    ];
                }
                catch (e) {
                    this.env.log.warn(`Could not load schematics for ${chalk_1.default.green('ionic generate')}. Use ${chalk_1.default.green('--verbose')} to debug.`);
                    debug(`Error while loading schematics: ${e.stack ? e.stack : e}`);
                    this.schematics = [];
                }
            }
            return this.schematics;
        });
    }
    generateComponent(schematic, name, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (schematic.collection !== ANGULAR_SCHEMATICS_PACKAGE) {
                options.collection = schematic.collection;
            }
            const ngArgs = cli_framework_1.unparseArgs(Object.assign({ _: ['generate', schematic.type, name] }, options), {});
            const shellOptions = { cwd: this.env.project.directory };
            const p = yield this.env.shell.spawn('ng', ngArgs, shellOptions);
            return new Promise((resolve, reject) => {
                let errorsEncountered = false;
                p.on('error', err => {
                    reject(err);
                });
                p.on('exit', code => {
                    if (code === 0) {
                        if (errorsEncountered) {
                            reject(new Error(`Angular CLI encountered errors while generating components.`));
                        }
                        else {
                            resolve();
                        }
                    }
                    else {
                        reject(new Error(`Angular CLI exited with error code: ${code}`));
                    }
                });
                process_1.onBeforeExit(() => tslib_1.__awaiter(this, void 0, void 0, function* () { return p.kill(); }));
                const log = this.env.log.clone({ wrap: false });
                const ws = log.createWriteStream();
                const stdoutStream = through2(function (chunk, env, callback) {
                    const str = chunk.toString();
                    if (str.includes('error!')) {
                        errorsEncountered = true;
                    }
                    this.push(chunk);
                    callback();
                });
                p.stdout.pipe(split2()).pipe(stdoutStream).pipe(ws);
                p.stderr.pipe(split2()).pipe(ws);
            });
        });
    }
}
exports.GenerateRunner = GenerateRunner;
